@@ -4,9 +4,10 @@ import sys
 import os
 import uuid
 import shutil
-from Bio import SeqIO
+#from os.path import join as opj
+
+from Bio import SeqIO, pairwise2
 from Bio.SeqRecord import SeqRecord
-from Bio import pairwise2
 from Bio.Seq import Seq
 
 try:
@@ -28,7 +29,7 @@ DEBUG = False
 class ModellerManager():
     """ Class to handle Modeller calculations """
     def __init__(self):
-        self.tmpdir = TMP_BASE_DIR + "/mod" + str(uuid.uuid4())
+        self.tmpdir = TMP_BASE_DIR +  "/mod" + str(uuid.uuid4())
         #self.tmpdir = "/tmp/modtest"
         #print("Using temporary working dir " + self.tmpdir)
         self.ch_id = ''
@@ -38,14 +39,24 @@ class ModellerManager():
             os.mkdir(self.tmpdir)
         except IOError as err:
             sys.exit(err)
-        self.env = environ()
+        # Class changed in Modeller >= 10
+        try:
+            self.env = Environ()
+        except:
+            self.env = environ()
+
         self.env.io.atom_files_directory = [self.tmpdir]
         log.none()
 
     def build(self, target_model, target_chain, extra_NTerm_res):
         """ Prepares Modeller input and builds the model """
         alin_file = self.tmpdir + "/alin.pir"
+
+        if not self.sequences.has_canonical[target_chain]:
+            raise NoCanSeqError(target_chain)
+
         tgt_seq = self.sequences.data[target_chain]['can'].seq
+
         #triming N-term of canonical seq
         pdb_seq = self.sequences.data[target_chain]['pdb'][target_model]['frgs'][0].seq
         nt_pos = max(tgt_seq.find(pdb_seq) - extra_NTerm_res, 0)
@@ -82,7 +93,7 @@ class ModellerManager():
                         frgs[-1].features[0].location.end,
                         ch_id
                     ),
-                    annotations = {'molecule_type':'protein'} # required for writing PIR aligment
+                    annotations={'molecule_type':'protein'} # required for writing PIR aligment
                 )
             )
             knowns.append('templ' + ch_id)
@@ -128,9 +139,13 @@ def _write_alin(tgt_seq, templs, alin_file):
                 'target',
                 '',
                 'sequence:target:::::::0.00: 0.00',
-                annotations = {'molecule_type':'protein'}
+                annotations={'molecule_type':'protein'}
             )
         ] + templs,
         alin_file,
         'pir'
     )
+
+class NoCanSeqError(Exception):
+    def __init__(self, ch_id):
+        self.message = 'No canonical sequence found for chain {}. Check it is defined in FASTA header (>anyId_A,B,...) or use mmCif input'.format(ch_id)
